#ifndef HSV_COLOR_HPP
#define HSV_COLOR_HPP

#include <math.h>
#include "cuda.hpp"
#include "rgbcolor.hpp"

HOST_DEV static inline RGBColor HSVtoRGB(float h, float s, float v) {
   RGBColor result;
   if (s == 0) {
      // achromatic (grey)
      return RGBColor(v, v, v);
   }
   h /= 60;
   int i = floor(h);
   float f = h - i;
   float p = v * (1 - s);
   float q = v * (1 - s * f);
   float t = v * (1 - s * (1 - f));

   switch (i) {
      case 0: return RGBColor(v, t, p);
      case 1: return RGBColor(q, v, p);
      case 2: return RGBColor(p, v, t);
      case 3: return RGBColor(p, q, v);
      case 4: return RGBColor(t, p, v);
      default: return RGBColor(v, p, q);
   }
}

struct HSVColor {
   HOST_DEV HSVColor() : hue(0), saturation(0), value(0) {
   }
   HOST_DEV HSVColor(float h, float s, float v) : hue(h), saturation(s), value(v) {
   }
   HOST_DEV RGBColor get_rgb() const {
      return HSVtoRGB(hue, saturation, value);
   }
   HOST_DEV HSVColor interpolate(const HSVColor& other, float t) {
      return HSVColor(
	 (other.hue == hue &&
	    other.saturation == saturation && other.value == value?
	       fmod(hue + (360 + other.hue - hue) * t, 360)
	    :
	       hue + (other.hue - hue) * t),
	 saturation + (other.saturation - saturation) * t,
	 value + (other.value - value) * t
      );
   }
   float hue;
   float saturation;
   float value;
};

static inline std::ostream& operator<<(std::ostream& out, const HSVColor& color) {
   return out << "(" << color.hue << ", " << color.saturation <<
      ", " << color.value << ")";
}

#endif
