#ifndef RGBCOLOR_H
#define RGBCOLOR_H

#include <iostream>
#include <cfloat>
#include <cmath>
#include "cuda.hpp"

struct RGBColor {
   HOST_DEV RGBColor() : red(0), green(0), blue(0) {
   }
   HOST_DEV RGBColor(float red_, float green_, float blue_) :
      red(red_), green(green_), blue(blue_) {
   }
   HOST_DEV RGBColor get_rgb() const {
      return *this;
   }
   HOST_DEV RGBColor interpolate(const RGBColor& other, float t) {
      return RGBColor(
	 (red + (other.red - red) * t),
	 (green + (other.green - green) * t),
	 (blue + (other.blue - blue) * t));
   }
   HOST_DEV bool operator==(const RGBColor& other) {
      return fabs(red - other.red) < DBL_EPSILON &&
             fabs(green - other.green) < DBL_EPSILON &&
	     fabs(blue - other.blue) < DBL_EPSILON;
   }
   HOST_DEV bool operator!=(const RGBColor& other) {
      return !(*this == other);
   }
   float red;
   float green;
   float blue;
};

static inline std::ostream& operator<<(std::ostream& out, const RGBColor& color) {
   return out << "(" << color.red << ", " << color.green <<
      ", " << color.blue << ")";
}

#endif
